#if defined __ze_vips_included
	#endinput
#endif
#define __ze_vips_included

#include <ze_vips_const>

/**
 * Check If player is VIP.
 *
 * @note If player has VIP_Z flags, This native will return value False.
 *
 * @param id     Client index.
 *
 * @return       true  | If player is VIP.
 *               false | If player is not VIP or player not on game.
 */
native ze_is_user_vip(id);

/**
 * Gets VIP flags of specific player.
 *
 * @note For a list of possible flags, see the VIP_* constants in ze_vips_const.inc
 *
 * @param id     Client index.
 *
 * @return       Bitflag sum of client's VIP flags, NULLENT on failure.
 * @error        If this player not on game.
 */
native ze_get_vip_flags(id);

/**
 * Sets VIP flags for specific player.
 *
 * @note For a list of possible flags, see the VIP_* constants in ze_vips_const.inc
 *
 * @param id     Client index.
 * @param iFlags VIP flags (VIP_*).
 *
 * @return       true  | If Flags assigned successfully.
 *               false | If player not on game.
 *
 * @error        If this player not on game.
 */
native ze_set_vip_flags(id, iFlags);

/**
 * Removes VIP flags for specific player.
 *
 * @param id     Client index.
 *
 * @return       true  | If Flags removed successfully.
 *               false | If player not on game.
 *
 * @error        If this player not on game.
 */
native ze_remove_vip_flags(id);

/**
 * Gets VIP expire date for specific player.
 *
 * @param id     Client index.
 *
 * @return       Unix timestamp on success,
 *               NULLENT (-1) If VIP is permanently or player not on game.
 *
 * @error        If this player not on game.
 */
native ze_get_vip_exDate(id);