#include <amxmodx>
#include <amxmisc>
#include <ze_core>
#include <ze_vips_const>
#include <ze_smooth_move>

// Defines 'VIP Features':
#define CONNECT_MESSAGE    VIP_A
#define MULTI_JUMPS        VIP_B
#define PARACHUTE          VIP_C
#define SCOREBOARD         VIP_D
#define MULTIPLE_DAMAGE    VIP_E
#define NO_FALL_DAMAGE     VIP_F
#define MULTI_GRENADES     VIP_G
#define CUSTOM_HP_SP       VIP_H
#define CUSTOM_MODEL       VIP_I
#define VIP_USER           VIP_Z

#define EXPIRE_NULL -1

#define CVAR_MAX_GRENADES  255

const MAX_PASSWORD_LENGTH = 32

enum _:VIP_DATA
{
	VIP_AUTHTYPE = 0, // Login auth type.
	VIP_AUTH[MAX_AUTHID_LENGTH],
	VIP_PASSWORD[MAX_PASSWORD_LENGTH],
	VIP_FLAGS,
	VIP_EXPIRE_DATE[16] // unix time.
}

enum _:Nades
{
	Frost = 0,
	Fire,
	Flare
}

// Cvars.
new g_iParachute,
	g_iMultiJumps,
	g_msgScoreAttrib,
	bool:g_bWhoOnline,
	bool:g_bConnectMsg,
	bool:g_bScoreBoard,
	bool:g_bNoFallDamage,
	Float:g_flMultiDamage,
	Float:g_flCustomHealth,
	Float:g_flCustomShield,
	g_szNades[CVAR_MAX_GRENADES],
	g_szField[32]

// Variables.
new g_iUserNum

// Array.
new g_iExpire[MAX_PLAYERS+1],
	g_bitsFlags[MAX_PLAYERS+1]

// String.
new g_szPassword[MAX_PLAYERS+1][MAX_PASSWORD_LENGTH]

// Dynamic Arrays.
new Array:g_aUsers,
	Array:g_aHumanVIPModels,
	Array:g_aZombieVIPModels

public plugin_natives()
{
	register_library("ze_vips")
	register_native("ze_is_user_vip", "__native_is_user_vip")
	register_native("ze_get_vip_flags", "__native_get_vip_flags")
	register_native("ze_set_vip_flags", "__native_set_vip_flags")
	register_native("ze_remove_vip_flags", "__native_remove_vip_flags")
	register_native("ze_get_vip_exDate", "__native_get_vip_exDate")
}

public plugin_precache()
{
	// Default VIP model.
	new const szHumanVIPModel[][] = { "vip" }
	new const szZombieVIPModel[][] = { "ze_regular_zombi" }

	// Create dyn Array.
	g_aHumanVIPModels = ArrayCreate(MAX_NAME_LENGTH, 1)
	g_aZombieVIPModels = ArrayCreate(MAX_NAME_LENGTH, 1)

	// Read VIP Model from INI file.
	ini_read_string_array(ZE_FILENAME, "Player Models", "VIP_HUMAN", g_aHumanVIPModels)
	ini_read_string_array(ZE_FILENAME, "Player Models", "VIP_ZOMBIE", g_aZombieVIPModels)

	new i

	if (!ArraySize(g_aHumanVIPModels))
	{
		for (i = 0; i < sizeof(szHumanVIPModel); i++)
			ArrayPushString(g_aHumanVIPModels, szHumanVIPModel[i])

		// Write VIP Model on INI file.
		ini_write_string_array(ZE_FILENAME, "Player Models", "VIP_HUMAN", g_aHumanVIPModels)
	}

	if (!ArraySize(g_aZombieVIPModels))
	{
		for (i = 0; i < sizeof(szZombieVIPModel); i++)
			ArrayPushString(g_aZombieVIPModels, szZombieVIPModel[i])

		// Write VIP Model on INI file.
		ini_write_string_array(ZE_FILENAME, "Player Models", "VIP_ZOMBIE", g_aZombieVIPModels)
	}

	new szPlayerModel[MAX_NAME_LENGTH], szModel[MAX_RESOURCE_PATH_LENGTH], iSizeNum

	iSizeNum = ArraySize(g_aHumanVIPModels)

	for (i = 0; i < iSizeNum; i++)
	{
		ArrayGetString(g_aHumanVIPModels, i, szPlayerModel, charsmax(szPlayerModel))

		// Precache Models
		formatex(szModel, charsmax(szModel), "models/player/%s/%s.mdl", szPlayerModel, szPlayerModel)
		precache_model(szModel)
	}

	iSizeNum = ArraySize(g_aZombieVIPModels)

	for (i = 0; i < iSizeNum; i++)
	{
		ArrayGetString(g_aZombieVIPModels, i, szPlayerModel, charsmax(szPlayerModel))

		// Precache Models
		formatex(szModel, charsmax(szModel), "models/player/%s/%s.mdl", szPlayerModel, szPlayerModel)
		precache_model(szModel)
	}
}

public plugin_init()
{
	// Load Plug-In.
	register_plugin("[ZE] VIPs System (Rebuild Only)", VIP_VERSION, ZE_AUTHORS)

	// Hook Chain.
	RegisterHookChain(RG_CBasePlayer_TakeDamage, "fw_TakeDamage_Pre")

	// Cvars.
	bind_pcvar_num(register_cvar("ze_vips_connectmsg", "1"), g_bConnectMsg)
	bind_pcvar_num(register_cvar("ze_vips_who_online", "1"), g_bWhoOnline)
	bind_pcvar_num(register_cvar("ze_vips_parachute", "1"), g_iParachute)
	bind_pcvar_num(register_cvar("ze_vips_multijumps", "1"), g_iMultiJumps)
	bind_pcvar_num(register_cvar("ze_vips_scoreboard", "1"), g_bScoreBoard)
	bind_pcvar_num(register_cvar("ze_vips_nofalldmg", "1"), g_bNoFallDamage)
	bind_pcvar_float(register_cvar("ze_vips_multidamage", "1.2"), g_flMultiDamage)
	bind_pcvar_float(register_cvar("ze_vips_health", "300.0"), g_flCustomHealth)
	bind_pcvar_float(register_cvar("ze_vips_shield", "100.0"), g_flCustomShield)
	bind_pcvar_string(register_cvar("ze_vips_grenades", "f"), g_szNades, charsmax(g_szNades))
	bind_pcvar_string(register_cvar("amx_password_field", "_pw"), g_szField, charsmax(g_szField)) // amxx.cfg

	// Commands.
	register_clcmd("say /vips", "cmd_WhoIsOnline")
	register_clcmd("say_team /vips", "cmd_WhoIsOnline")

	// Create dyn Array.
	g_aUsers = ArrayCreate(VIP_DATA, 1)

	// Initial Value.
	g_msgScoreAttrib = get_user_msgid("ScoreAttrib")
}

public plugin_cfg()
{
	new szFile[64], szCfgDir[24]
	get_configsdir(szCfgDir, charsmax(szCfgDir))
	formatex(szFile, charsmax(szFile), "%s/%s.ini", szCfgDir, VIP_FILENAME)

	// Read VIP users from INI file.
	if (read_VIPs(szFile))
		server_print("[ZE][VIPs System] %i Users has been loaded from %s.ini", g_iUserNum, VIP_FILENAME)
}

public plugin_end()
{
	ArrayDestroy(g_aUsers)
	ArrayDestroy(g_aHumanVIPModels)
	ArrayDestroy(g_aZombieVIPModels)
}

public cmd_WhoIsOnline(const id)
{
	if (!g_bWhoOnline)
		return PLUGIN_CONTINUE

	new szMessage[256], iClients[MAX_PLAYERS], iClientNum, iLen, iNum
	get_players(iClients, iClientNum, "h")

	// Add players name to buffer.
	for (new szName[MAX_NAME_LENGTH], iClient, i = 0; i < iClientNum; i++)
	{
		iClient = iClients[i]

		// Player is VIP?
		if (g_bitsFlags[iClient] > VIP_0 && !(g_bitsFlags[iClient] & VIP_Z))
		{
			iNum++
			get_user_name(iClient, szName, charsmax(szName))
			iLen += formatex(szMessage[iLen], charsmax(szMessage) - iLen, "%s, ", szName)
		}
	}

	// Send colored message on chat for everyone.
	if (iNum)
	{
		szMessage[iLen - 2] = '.'
		ze_colored_print(0, "%L", LANG_PLAYER, "MSG_VIP_ONLINE", szMessage)
	}
	else
	{
		ze_colored_print(0, "%L", LANG_PLAYER, "MSG_VIP_OFFLINE")
	}

	return PLUGIN_CONTINUE
}

public client_authorized(id, const szAuthID[])
{
	// HLTV Proxy?
	if (is_user_hltv(id))
		return

	// Get player's name.
	new szName[MAX_NAME_LENGTH]
	get_user_name(id, szName, charsmax(szName))

	// check If player is VIP or not.
	if (!check_Client(id, szName, szAuthID))
		g_iExpire[id] = EXPIRE_NULL
}

public client_infochanged(id)
{
	// Player disconnected or is HLTV?
	if (!is_user_connected(id) || is_user_hltv(id))
		return

	new szName[MAX_NAME_LENGTH], szNewName[MAX_NAME_LENGTH], szPassword[MAX_PASSWORD_LENGTH]
	get_user_name(id, szName, charsmax(szName))
	get_user_info(id, "name", szNewName, charsmax(szNewName))
	get_user_info(id, g_szField, szPassword, charsmax(szPassword))

	// Same name?
	if (equal(szName, szNewName) || (g_szPassword[id][0] && equal(szPassword, g_szPassword[id])))
		return

	// Get player's SteamID.
	new szAuthID[MAX_AUTHID_LENGTH]
	get_user_authid(id, szAuthID, charsmax(szAuthID))

	// Check If player is VIP or not?
	if (!check_Client(id, szNewName, szAuthID, false))
	{
		g_bitsFlags[id] = VIP_0
		g_iExpire[id]   = EXPIRE_NULL
	}
}

public client_disconnected(id, bool:drop, message[], maxlen)
{
	// HLTV Proxy?
	if (is_user_hltv(id))
		return

	g_iExpire[id] = 0
	g_bitsFlags[id] = 0
	g_szPassword[id] = NULL_STRING
}

public VIP_Connected(id, const szName[])
{
	if (g_bConnectMsg)
	{
		// Send colored message on chat for everyone.
		ze_colored_print(id, "%L", LANG_PLAYER, "MSG_VIP_CONNECTED", szName)
	}
}

public ze_game_started()
{
	new iClients[MAX_PLAYERS], iCount
	get_players(iClients, iCount, "h")

	for (new id, i = 0; i < iCount; i++)
	{
		id = iClients[i]

		// Expired?
		if (g_iExpire[id] == EXPIRE_NULL)
			continue

		if (VIP_Expired(id))
		{
			// Send colored message on chat for player.
			ze_colored_print(id, "%L", LANG_PLAYER, "MSG_VIP_EXPIRED")
		}
	}
}

public ze_user_humanized(id)
{
	if ( g_bitsFlags[id] & MULTI_JUMPS )
	{
		switch (g_iMultiJumps)
		{
			case 1, 2: // All, Human.
			{
				ze_set_user_multijump(id, true)
			}
			case 3: // Zombie only.
			{
				ze_set_user_multijump(id, false)
			}
		}
	}

	if (g_szNades[0] != EOS && ( g_bitsFlags[id] & MULTI_GRENADES ))
	{
		new iGrenades[Nades]

		// Get grenades amount from cvar.
		iGrenades[Frost] = charCount(g_szNades, 'f')
		iGrenades[Fire] = charCount(g_szNades, 'h')
		iGrenades[Flare] = charCount(g_szNades, 's')

		if (iGrenades[Frost])
		{
			if (!rg_get_user_bpammo(id, WEAPON_FLASHBANG))
				rg_give_item(id, "weapon_flashbang", GT_APPEND)
			rg_set_user_bpammo(id, WEAPON_FLASHBANG, iGrenades[Frost])
		}

		if (iGrenades[Fire])
		{
			if (!rg_get_user_bpammo(id, WEAPON_HEGRENADE))
				rg_give_item(id, "weapon_hegrenade", GT_APPEND)
			rg_set_user_bpammo(id, WEAPON_HEGRENADE, iGrenades[Fire])
		}

		if (iGrenades[Flare])
		{
			if (!rg_get_user_bpammo(id, WEAPON_SMOKEGRENADE))
				rg_give_item(id, "weapon_smokegrenade", GT_APPEND)
			rg_set_user_bpammo(id, WEAPON_SMOKEGRENADE, iGrenades[Flare])
		}
	}

	if ( g_bitsFlags[id] & CUSTOM_MODEL )
	{
		new szModel[MAX_NAME_LENGTH]
		ArrayGetString(g_aHumanVIPModels, random_num(0, ArraySize(g_aHumanVIPModels) - 1), szModel, charsmax(szModel))
		rg_set_user_model(id, szModel, true)
	}

	if ( g_bitsFlags[id] & CUSTOM_HP_SP )
	{
		if (g_flCustomHealth > 0.0)
		{
			set_entvar(id, var_health, g_flCustomHealth)
			set_entvar(id, var_max_health, g_flCustomHealth)
		}

		if (g_flCustomShield > 0.0)
		{
			set_entvar(id, var_armortype, ARMOR_VESTHELM)
			set_entvar(id, var_armorvalue, g_flCustomShield)
			//rg_set_user_armor(id, float(g_flCustomShield), ARMOR_VESTHELM)
		}
	}
}

public ze_user_infected(iVictim, iInfector)
{
	if ( g_bitsFlags[iVictim] & MULTI_JUMPS )
	{
		switch (g_iMultiJumps)
		{
			case 1, 3: // All, Zombie.
			{
				ze_set_user_multijump(iVictim, false)
			}
			case 2: // Human only.
			{
				ze_set_user_multijump(iVictim, true)
			}
		}
	}

	if (g_bScoreBoard && ( g_bitsFlags[iVictim] & SCOREBOARD ))
	{
		message_begin(MSG_ALL, g_msgScoreAttrib, _, iVictim)
		write_byte(iVictim) // Client index.
		write_byte(4) // 4 = VIP Attrib.
		message_end()
	}

	if ( g_bitsFlags[iVictim] & CUSTOM_MODEL )
	{
		new szModel[MAX_NAME_LENGTH]
		ArrayGetString(g_aZombieVIPModels, random_num(0, ArraySize(g_aZombieVIPModels) - 1), szModel, charsmax(szModel))
		rg_set_user_model(iVictim, szModel, true)
	}
}

public ze_user_spawn_post(id)
{
	if (g_bScoreBoard && ( g_bitsFlags[id] & SCOREBOARD ))
	{
		message_begin(MSG_ALL, g_msgScoreAttrib, _, id)
		write_byte(id) // Client index.
		write_byte(4) // 4 = VIP Attrib.
		message_end()
	}
}

public fw_TakeDamage_Pre(iVictim, iInflector, iAttacker, Float:flDamage, bitsDamageType)
{
	if (( g_flMultiDamage != 1.0 ) && iVictim != iAttacker  && is_user_connected(iAttacker) && ( g_bitsFlags[iAttacker] & MULTIPLE_DAMAGE ))
	{
		// Multiple damage.
		SetHookChainArg(4, ATYPE_FLOAT, flDamage * g_flMultiDamage)
		return HC_CONTINUE
	}

	if (g_bNoFallDamage && is_user_connected(iVictim) && iVictim == iAttacker && ( g_bitsFlags[iVictim] & NO_FALL_DAMAGE ) && ( bitsDamageType & DMG_FALL ))
	{
		// Prevent Fall damage.
		SetHookChainReturn(ATYPE_INTEGER, HC_SUPERCEDE)
		return HC_SUPERCEDE
	}

	return HC_CONTINUE
}

public VIP_Expired(const id)
{
	if (g_iExpire[id] - get_systime() < 1)
	{
		g_bitsFlags[id] = VIP_0
		g_iExpire[id] = EXPIRE_NULL
		return true
	}

	return false
}

/**
 * -=| Functions |=-
 */
read_VIPs(const szFile[])
{
	new hFile // Open the file.
	if ((hFile = fopen(szFile, "rt")))
	{
		new szRead[256], szSection[64], szUserAuth[128], szPassKey[128], szVIPFlags[64], szExpireDate[32], iLine
		new _Array[VIP_DATA], cAuthType, iCurs, iTheTime = get_systime()

		while (!feof(hFile))
		{
			iLine++

			szRead = NULL_STRING
			if (!fgets(hFile, szRead, charsmax(szRead)))
				break

			trim(szRead), replace(szRead, charsmax(szRead), "^n", "")

			switch (szRead[0])
			{
				case '#', ';', 0: // Comment or empty line.
				{
					continue
				}
				case '[': // Section.
				{
					copyc(szSection, charsmax(szSection), szRead[1], ']')

					if (equal(szSection, "Name", 4))
						cAuthType = 'n'
					else if (equal(szSection, "Name CSens", 10))
						cAuthType = 'N'
					else if (equal(szSection, "AuthID", 6))
						cAuthType = 'a'
				}
			}

			if (cAuthType != 0)
			{
				while (!feof(hFile))
				{
					iLine++
					iCurs = ftell(hFile)
					szRead = NULL_STRING

					if (!fgets(hFile, szRead, charsmax(szRead)))
						break

					trim(szRead), replace(szRead, charsmax(szRead), "^n", "")

					switch (szRead[0])
					{
						case '#', ';', 0:
						{
							continue
						}
						case '[':
						{
							fseek(hFile, iCurs, SEEK_SET)
							break
						}
					}

					// Parse the text.
					if (parse(szRead, szUserAuth, charsmax(szUserAuth), szPassKey, charsmax(szPassKey), szVIPFlags, charsmax(szVIPFlags), szExpireDate, charsmax(szExpireDate)) < 3)
					{
						server_print("[ZE][VIPs System] Lost argument(s) in Line #%i", iLine)
						continue
					}

					// VIP Expired?
					if (szExpireDate[0] && ( parse_time(szExpireDate, "%H:%M %d-%m-%y") - iTheTime <= 0 ))
						continue

					_Array[VIP_AUTHTYPE] = cAuthType
					_Array[VIP_FLAGS] = read_flags(szVIPFlags)
					copy(_Array[VIP_AUTH], charsmax(_Array) - VIP_AUTH, szUserAuth)
					copy(_Array[VIP_PASSWORD], charsmax(_Array) - VIP_PASSWORD, szPassKey)
					copy(_Array[VIP_EXPIRE_DATE], charsmax(_Array) - VIP_EXPIRE_DATE, szExpireDate)

					ArrayPushArray(g_aUsers, _Array)
					g_iUserNum++
				}

				cAuthType = 0
			}
		}

		// Close the file.
		fclose(hFile)
		return 1
	}

	return 0
}

check_Client(id, const szName[], const szAuthID[], bool:bNew = true)
{
	for (new _Array[VIP_DATA], bool:bFound, i = 0; i < g_iUserNum; i++)
	{
		ArrayGetArray(g_aUsers, i, _Array)

		switch (_Array[VIP_AUTHTYPE])
		{
			case 'n':
				if (equali(szName, _Array[VIP_AUTH])) bFound = true
			case 'N':
				if (equal(szName, _Array[VIP_AUTH])) bFound = true
			case 'a':
				if (equali(szAuthID, _Array[VIP_AUTH])) bFound = true
		}

		if (bFound)
		{
			new szPassword[MAX_PASSWORD_LENGTH]
			get_user_info(id, g_szField, szPassword, charsmax(szPassword)) // Get player's password.

			if (_Array[VIP_PASSWORD] && !equal(szPassword, _Array[VIP_PASSWORD]))
			{
				// Kick the player.
				server_cmd("kick #%i ^"%L^"", get_user_userid(id), LANG_PLAYER, "NET_VIP_WRONG", g_szField)
				server_exec()
			}
			else // Authorized!
			{
				copy(g_szPassword[id], charsmax(g_szPassword[]), szPassword)
				g_iExpire[id] = _Array[VIP_EXPIRE_DATE] ? parse_time(_Array[VIP_EXPIRE_DATE], "%H:%M %d-%m-%y") : EXPIRE_NULL
				g_bitsFlags[id] = _Array[VIP_FLAGS]

				new szIP[MAX_IP_LENGTH]
				get_user_ip(id, szIP, charsmax(szIP))

				new szFlags[MAX_VIP_FLAGS]
				get_flags(_Array[VIP_FLAGS], szFlags, charsmax(szFlags))

				// Print message on server console.
				log_amx("Login: Name: ^'%s^' - AuthID ^'%s^' - IP address: ^'%s^' - Flags: ^'%s^'", szName, szAuthID, szIP, szFlags)

				if (bNew)
					VIP_Connected(id, szName)
			}

			return true // Stop searching.
		}
	}

	return false
}

charCount(const str[], ch)
{
	if (ch == 0 || str[0] == EOS)
		return 0

	new const size = strlen(str)

	new n
	for (new i = 0; i < size; i++)
	{
		if (str[i] == ch)
			n++
	}

	return n
}

/**
 * -=| Natives |=-
 */
public bool:__native_is_user_vip(const plugin_id, const num_params)
{
	static id; id = get_param(1)

	if (!is_user_connected(id))
		return false

	return (g_bitsFlags[id] > VIP_0 && !(g_bitsFlags[id] & VIP_Z))
}

public __native_get_vip_flags(const plugin_id, const num_params)
{
	static id; id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return NULLENT
	}

	return g_bitsFlags[id]
}

public __native_set_vip_flags(const plugin_id, const num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return false
	}

	g_bitsFlags[id] = get_param(2)
	return true
}

public __native_remove_vip_flags(const plugin_id, const num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return false
	}

	g_bitsFlags[id] = VIP_0
	g_iExpire[id] = EXPIRE_NULL
	return true
}

public __native_get_vip_exDate(const plugin_id, const num_params)
{
	new id = get_param(1)

	if (!is_user_connected(id))
	{
		log_error(AMX_ERR_NATIVE, "[ZE] Player not on game (%d)", id)
		return NULLENT
	}

	return g_iExpire[id]
}